import threading


class Proxy:
    def __init__(self, method, *args, **kwargs):
        assert callable(method), 'method must be a function'
        self.original_method = method
        self.init = kwargs.get("init")
        self.callback = kwargs.get("callback")

    def __call__(self, *args, **kwargs):
        return self.__new_method(*args, **kwargs)

    def __new_method(self, *args, **kwargs):
        if self.init:
            self.init()

        result = self.original_method(*args, **kwargs)

        from django.db import connections
        try:
            connections.close_all()
        except:
            pass

        if self.callback:
            if self.callback.__code__.co_argcount == 1:
                self.callback(result)
            else:
                self.callback()

        return result

    def delay(self, *args, **kwargs):
        t = threading.Thread(target=self.__new_method, args=args, kwargs=kwargs)
        t.daemon = True
        t.start()

    def execute(self, *args, **kwargs):
        self.__new_method(*args, **kwargs)


def task(*args, **kwargs):
    if len(args) == 1 and callable(args[0]):
        return Proxy(args[0])

    def wrapper(method):
        return Proxy(method, *args, **kwargs)

    return wrapper



import json
import threading

from django import urls
from django.http import HttpResponse


class R(HttpResponse):
    def __init__(self, code=10000, message='', description='', *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.code = code
        self.message = message
        self.description = description
        self.content_type = "application/json"
        self.content = self.__str__()

    def __str__(self):
        result = {
            'code': self.code,
            'message': self.message,
            'description': self.description
        }
        return json.dumps(result)

    @classmethod
    def success(cls):
        return R(message='success')

    @classmethod
    def fail(cls, code=9999, description=None):
        return R(message='fail', code=code, description=description)


class Proxy:

    def __init__(self, method, *args, **kwargs):
        assert callable(method), 'method must be a function'
        self.original_method = method
        self.init = kwargs.get("init")
        self.callback = kwargs.get("callback")

    def __new_method(self, *args, **kwargs):
            if self.init:
                self.init()

            self.original_method()

            from django.db import connections
            connections.close_all()

            if self.callback:
                self.callback()

    def delay(self, *args, **kwargs):
        try:
            t = threading.Thread(target=self.__new_method, args=args, kwargs=kwargs)
            t.daemon = True
            t.start()
        except Exception as e:
            return R.fail(description=str(e))
        return R.success()

    def execute(self, *args, **kwargs):
        try:
            self.__new_method(*args, **kwargs)
        except Exception as e:
            return R.fail(description=str(e))
        return R.success()


class Route:
    def __init__(self, url_prefix='', name_prefix=''):
        self.url_prefix = url_prefix.strip('/')
        self.name_prefix = name_prefix
        self.routes = {}

    def __call__(self, path=None, name=None, *args, **kwargs):

        def wrapper(method):
            url_name = self.name_prefix + (name or method.__name__)
            url_path = path or method.__name__
            url_path = self.url_prefix + '/' + url_path.lstrip('/')
            self.routes[url_name] = url_path, Proxy(method, *args, **kwargs).delay
            return Proxy(method, *args, **kwargs).delay
        if callable(path):
            method = path
            path = None
            return wrapper(method)
        return wrapper

    @property
    def patterns(self):
        return [urls.path(path.lstrip('/'), handler, name=name)
                for name, (path, handler) in self.routes.items()]

    @property
    def names(self):
        return {name: '/' + path.lstrip('/')
                for name, (path, _) in self.routes.items()}



#!/usr/bin/env python
# -*- coding:utf-8 -*-

#############################################
# File Name: setup.py
# Author: camellibby
# Mail: camelxinliang@sina.com
# Created Time: 2019-11-22
#############################################


from setuptools import setup, find_packages

setup(
    name="pangu-task",
    version="0.6.2",
    keywords=("pip", "python", "pangu", "job",  "task"),
    description="Asynchronously run function with callback",
    long_description="Asynchronously run function with callback",
    license="MIT Licence",

    url="https://github.com/camellibby/pangu-task",
    author="camellibby",
    author_email="camelxinliang@sina.com",

    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=[]
)
